@@ -19,18 +19,19 @@ Attr_Access_a = namedtuple('Attr_Access_a', 'cname its attr')
 Rank_Selection_a = namedtuple('Rank_Selection_a', 'card rankr attr')
 Criteria_Selection_a = namedtuple('Criteria_Selection_a', 'card criteria')
 Inst_Assignment_a = namedtuple('Inst_Assignment_a', 'lhs card rhs X')
-EE_Signal_a = namedtuple('EE_Signal_a', 'event supplied_params ee')
+External_Signal_a = namedtuple('External_Signal_a', 'event supplied_params')
 Signal_a = namedtuple('Signal_a', 'event supplied_params dest')
 """Signal sent to trigger event at destination with optional supplied parameters"""
-Signal_Action_a = namedtuple('Signal_Action_a', 'event supplied_params dest delay assigner_partition')
-Signal_Dest_a = namedtuple('Signal_Dest_a', 'target_iset assigner_partition delay')
+Signal_Action_a = namedtuple('Signal_Action_a', 'event supplied_params dest delay')
+Signal_Dest_a = namedtuple('Signal_Dest_a', 'target_iset assigner_dest delay cancel')
+Assigner_Dest_a = namedtuple('Assigner_Dest_a', 'rnum partition')
 Signal_Choice_a = namedtuple('Signal_Choice_a', 'decision true_signal false_signal')
 Sequence_Token_a = namedtuple('Sequence_Token_a', 'name')
 Execution_Unit_a = namedtuple('Execution_Unit_a', 'statement_set output_token')
 Seq_Statement_Set_a = namedtuple('Seq_Statement_Set_a', 'input_tokens statement block')
 Comp_Statement_Set_a = namedtuple('Comp_Statement_Set_a', 'statement block')
 Decision_a = namedtuple('Decision_a', 'input true_result false_result')
-Delete_Action_a = namedtuple('Delete_Action_a', 'instance_sets')
+Delete_Group_a = namedtuple('Delete_Group_a', 'instance_sets')
 Case_a = namedtuple('Case_a', 'enums comp_statement_set')
 Switch_a = namedtuple('Switch_a', 'input_flow cases')
 MATH_a = namedtuple('MATH_a', 'op operands')
@@ -646,13 +647,13 @@ class ScrallVisitor(PTNodeVisitor):
     def visit_signal_action(cls, node, children):
         """
         """
-        _logger.info("signal_action = signal_spec SP+ (signal_dest / ee_dest / SIGNAL_OP)")
+        _logger.info("signal_action = signal_spec SP+ (signal_dest / external_dest / SIGNAL_OP)")
         _logger.info(f'  :: {node.value}')
 
         _logger.info(f"  < {children}")
         sdest = children.results.get('signal_dest')
-        eedest = children.results.get('ee_dest')
-        if not sdest and not eedest:
+        external_dest = bool(children.results.get('external_dest'))
+        if not sdest and not external_dest:
             # Dest should be supplied by false result in decision
             result = Signal_a(
                 event=children[0]['name'],
@@ -667,11 +668,10 @@ class ScrallVisitor(PTNodeVisitor):
                 dest=children[1]
             )
         else:
-            # Otherwise it must be an EE destination
-            result = EE_Signal_a(
+            # Otherwise it must be an external destination
+            result = External_Signal_a(
                 event=children[0]['name'],
                 supplied_params=children[0]['params'],
-                ee=children[1]
             )
 
         _logger.info(f"  > {result}")
@@ -691,14 +691,14 @@ class ScrallVisitor(PTNodeVisitor):
         return result
 
     @classmethod
-    def visit_ee_dest(cls, node, children):
+    def visit_external_dest(cls, node, children):
         """
         """
-        _logger.info("ee_dest = ASYNCH name")
+        _logger.info("external_dest = SIGNAL_OP LINEWRAP? SP? '~'")
         _logger.info(f'  :: {node.value}')
 
         _logger.info(f"  < {children}")
-        result = children[0]
+        result = '~'
         _logger.info(f"  > {result}")
         return result
 
@@ -706,16 +706,33 @@ class ScrallVisitor(PTNodeVisitor):
     def visit_signal_dest(cls, node, children):
         """
         """
-        _logger.info("signal_action = signal_spec SP+ (signal_dest / ee_dest / SIGNAL_OP / ASYNCH)")
+        _logger.info("signal_dest = SIGNAL_OP LINEWRAP? SP? (assigner / instance_set) (SP+ delay)?")
+        _logger.info(f'  :: {node.value}')
+        cancel = bool(children[0] == '*')
+
+        _logger.info(f"  < {children}")
+        iset = children.results.get('instance_set')
+        iset = iset[0] if iset else None
+        assigner = children.results.get('assigner')
+        assigner = assigner[0] if assigner else None
+        delay = children.results.get('delay')
+        delay = 0 if not delay else delay[0]
+        result = Signal_Dest_a(target_iset=iset, assigner_dest=assigner, delay=delay, cancel=cancel)
+        _logger.info(f"  > {result}")
+        return result
+
+    @classmethod
+    def visit_assigner(cls, node, children):
+        """
+        An instance set that partitions an assigner
+        """
+        _logger.info("assigner = rnum assigner_partition?")
         _logger.info(f'  :: {node.value}')
 
         _logger.info(f"  < {children}")
-        iset = children[0]
+        rnum = children.results.get('rnum')
         ap = children.results.get('assigner_partition')
-        ap = None if not ap else ap[0]
-        delay = children.results.get('delay')
-        delay = 0 if not delay else delay[0]
-        result = Signal_Dest_a(target_iset=iset, assigner_partition=N_a(ap), delay=delay)
+        result = Assigner_Dest_a(rnum=rnum[0], partition=ap[0] if ap else None)
         _logger.info(f"  > {result}")
         return result
 
@@ -724,7 +741,7 @@ class ScrallVisitor(PTNodeVisitor):
         """
         An instance set that partitions an assigner
         """
-        _logger.info("assigner_partition = '(' instance_set ')'")
+        _logger.info("assigner_partition = '( ' instance_set ' )'")
         _logger.info(f'  :: {node.value}')
 
         _logger.info(f"  < {children}")
@@ -791,14 +808,14 @@ class ScrallVisitor(PTNodeVisitor):
         """
         Post-parse verify that last element is an operation, otherwise invalid call
         """
-        _logger.info("call = instance_set op_chain?")
+        _logger.info("call = ('~' / instance_set) op_chain?")
         _logger.info(f'  :: {node.value}')
 
         _logger.info(f"  < {children}")
-        iset = children.results['instance_set'][0]
+        iset = children.results.get('instance_set')
         opc = children.results.get('op_chain')
         result = Call_a(
-            call=iset,
+            call=iset[0] if iset else None,  # None means that we are calling a service mapped to a service domain
             op_chain=None if not opc else opc[0]
         )
         _logger.info(f"  > {result}")
@@ -916,7 +933,7 @@ class ScrallVisitor(PTNodeVisitor):
         by any sequence of selection, operation, and paths. The parser won't find two paths in sequence since
         any encounter path will be fully consumed
         """
-        _logger.info("instance_set = new_instance / ((operation / prefix_name / path) (reflexive_selection / "
+        _logger.info("instance_set = new_instance / ((operation / input_param / name / path) (reflexive_selection / "
                      "selection / operation / path)*)")
         _logger.info(f">> {[k for k in children.results.keys()]}")
         _logger.info(f'  :: {node.value}')
@@ -925,7 +942,12 @@ class ScrallVisitor(PTNodeVisitor):
         if len(children) == 1 and isinstance(children[0], N_a):
             result = children[0]
         else:
-            result = INST_a(children)
+            p = children.results.get('input_param')
+            if p:
+                # Just like above case, but returning an IN_a (parameter name)
+                result = p[0]
+            else:
+                result = INST_a(children)
         _logger.info(f"  > {result}")
         return result
 
@@ -1008,7 +1030,7 @@ class ScrallVisitor(PTNodeVisitor):
         _logger.info(f"  < {children}")
         a = children.results.get('attr_init')
         r = children.results.get('to_ref')
-        result = New_inst_a(cname=children[0], attrs=None if not a else a[0], rels=None if not r else r[0])
+        result = New_inst_a(cname=children[0], attrs=a[0] if a else [], rels=None if not r else r)
         _logger.info(f"  > {result}")
         return result
 
@@ -1077,7 +1099,7 @@ class ScrallVisitor(PTNodeVisitor):
 
         _logger.info(f"  < {children}")
         iset = children.results.get('instance_set')
-        result = Delete_Action_a(instance_sets=iset)
+        result = Delete_Group_a(instance_sets=iset)
         _logger.info(f"  > {result}")
         return result
 
